| Safe Haskell | None |
|---|---|
| Language | GHC2021 |
Language.QBE.Analysis.CDG
Description
This module implements a control dependency analysis, using a control dependency graph (CDG) for more information on the concept refer to https://doi.org/10.1145/24039.24041. Roughly speaking, a node A is control dependent on B if there is an edge B → A so that the node is taken, as well as an edge so that it is not taken.